@@ -13,10 +13,10 @@
 #    under the License.
 
 import os
+from unittest import mock
 
 import ddt
 from eventlet import timeout as etimeout
-import mock
 from os_win import constants as os_win_const
 from os_win import exceptions as os_win_exc
 from oslo_concurrency import processutils
@@ -1827,7 +1827,7 @@ class VMOpsTestCase(test_base.HyperVBaseTestCase):
         self.assertEqual(fake_local_disks, ret_val)
 
     def test_get_scoped_flavor_extra_specs(self):
-        # The flavor extra spect dict contains only string values.
+        # The flavor extra specs dict contains only string values.
         fake_total_bytes_sec = '8'
 
         mock_instance = fake_instance.fake_instance_obj(self.context)
